import pytest
import sys

if sys.version_info < (3,10):
    pytest.skip("Skipping pyodide tests on older Python", allow_module_level=True)

from pytest_pyodide import run_in_pyodide

from itkwasm_htj2k_emscripten import __version__ as test_package_version

@pytest.fixture
def package_wheel():
    return f"itkwasm_htj2k_emscripten-{test_package_version}-py3-none-any.whl"

@run_in_pyodide(packages=['micropip'])
async def test_example(selenium, package_wheel):
    import micropip
    await micropip.install(package_wheel)

    # Write your test code here



import pickle

if __name__ == '__main__':
    from pathlib import Path
    input_base_path = Path('..', '..', 'test', 'data')
    test_files = [
        Path('input') / 'itk_logo.png',
        Path('input') / 'CT1.mha',
    ]
    data = {}
    import itk
    from itkwasm import Image
    for f in test_files:
        path = input_base_path / f
        image = itk.imread(path)
        image_dict = itk.dict_from_image(image)
        image_dict['metadata'] = dict()
        print(str(path))
        with open(f'{str(path)}.pickle', 'wb') as fp:
            # Pickle dict representation for use in pyodide test to avoid
            # bootstrapping conflict with itkwasm_image_io
            pickle.dump(image_dict, fp)


# Generated file. To retain edits, remove this comment.

from pathlib import Path
import os
from typing import Dict, Tuple, Optional, List, Any

from .js_package import js_package

from itkwasm.pyodide import (
    to_js,
    to_py,
    js_resources
)
from itkwasm import (
    InterfaceTypes,
    BinaryStream,
    Image,
)

async def decode_async(
    codestream: bytes,
    decomposition_level: int = 0,
    information_only: bool = False,
) -> Image:
    """Decode a High Throughput JPEG2000 codestream and generate an ITK Image

    :param codestream: Input HTJ2K codestream
    :type  codestream: bytes

    :param decomposition_level: Decomposition level to decode. 0 is the highest resolution.
    :type  decomposition_level: int

    :param information_only: Only read metadata for the image but do not decode the pixel data.
    :type  information_only: bool

    :return: Output image
    :rtype:  Image
    """
    js_module = await js_package.js_module
    web_worker = js_resources.web_worker

    kwargs = {}
    if decomposition_level:
        kwargs["decompositionLevel"] = to_js(decomposition_level)
    if information_only:
        kwargs["informationOnly"] = to_js(information_only)

    outputs = await js_module.decode(to_js(codestream), webWorker=web_worker, noCopy=True, **kwargs)

    output_web_worker = None
    output_list = []
    outputs_object_map = outputs.as_object_map()
    for output_name in outputs.object_keys():
        if output_name == 'webWorker':
            output_web_worker = outputs_object_map[output_name]
        else:
            output_list.append(to_py(outputs_object_map[output_name]))

    js_resources.web_worker = output_web_worker

    if len(output_list) == 1:
        return output_list[0]
    return tuple(output_list)



import pytest
import sys
import pickle

if sys.version_info < (3,10):
    pytest.skip("Skipping pyodide tests on older Python", allow_module_level=True)

from itkwasm_htj2k_emscripten import __version__ as test_package_version

@pytest.fixture
def package_wheel():
    return f"itkwasm_htj2k_emscripten-{test_package_version}-py3-none-any.whl"

@pytest.fixture
def input_data():
    from pathlib import Path
    input_base_path = Path('..', '..', 'test', 'data')
    test_files = [
        Path('input') / 'itk_logo.png',
        Path('input') / 'CT1.mha',
    ]
    data = {}
    for f in test_files:
        path = str(input_base_path / f) + '.pickle'
        with open(path, 'rb') as fp:
            data[str(f.name)] = pickle.load(fp)
    return data


import sys

if sys.version_info < (3,10):
    pytest.skip("Skipping pyodide tests on older Python", allow_module_level=True)

from pytest_pyodide import run_in_pyodide
from .fixtures import package_wheel, input_data

@run_in_pyodide(packages=['micropip', 'numpy'])
async def test_encode_decode_rgba_async(selenium, package_wheel, input_data):
    import micropip
    await micropip.install([package_wheel, 'numpy', 'itkwasm', 'itkwasm-compare-images'])

    from itkwasm_htj2k_emscripten import encode_async, decode_async
    from itkwasm_compare_images import compare_images_async
    import numpy as np
    from itkwasm import Image

    input_file = 'itk_logo.png'
    image = Image(**input_data[input_file])

    codestream = await encode_async(image, decompositions=5, tile_size=[256, 256], block_dimensions=[64, 64])
    decoded_image = await decode_async(codestream)

    metrics, _, _ = await compare_images_async(image, baseline_images=[decoded_image,])
    assert metrics['almostEqual'] == True


@run_in_pyodide(packages=['micropip', 'numpy'])
async def test_encode_decode_short_async(selenium, package_wheel, input_data):
    import micropip
    await micropip.install([package_wheel, 'numpy', 'itkwasm', 'itkwasm-compare-images'])

    from itkwasm_htj2k_emscripten import encode_async, decode_async
    from itkwasm_compare_images import compare_images_async
    import numpy as np
    from itkwasm import Image

    input_file = 'CT1.mha'
    image = Image(**input_data[input_file])

    codestream = await encode_async(image, decompositions=5, tile_size=[256, 256], block_dimensions=[64, 64])
    decoded_image = await decode_async(codestream)

    metrics, _, _ = await compare_images_async(image, baseline_images=[decoded_image,])
    assert metrics['almostEqual'] == True


# Generated file. To retain edits, remove this comment.

from pathlib import Path
import os
from typing import Dict, Tuple, Optional, List, Any

from .js_package import js_package

from itkwasm.pyodide import (
    to_js,
    to_py,
    js_resources
)
from itkwasm import (
    InterfaceTypes,
    Image,
    BinaryStream,
)

async def encode_async(
    image: Image,
    decompositions: int = 5,
    not_reversible: bool = False,
    quantization_step: float = 1,
    progression_order: int = 2,
    tile_size: Optional[List[int]] = None,
    tile_offset: Optional[List[int]] = None,
    block_dimensions: Optional[List[int]] = None,
) -> bytes:
    """Encode an ITK Image into a High Throughput JPEG2000 codestream

    :param image: Input image
    :type  image: Image

    :param decompositions: Number of wavelet decompositions
    :type  decompositions: int

    :param not_reversible: Lossy and not reversible, i.e. not lossless, encoding
    :type  not_reversible: bool

    :param quantization_step: Quantization step size for lossy compression. Ignored unless not reversible is enabled.
    :type  quantization_step: float

    :param progression_order: Progression order. 0 = LRCP, 1 = RLCP, 2 = RPCL, 3 = PCRL, 4 = CPRL
    :type  progression_order: int

    :param tile_size: Tile size
    :type  tile_size: int

    :param tile_offset: Tile offset
    :type  tile_offset: int

    :param block_dimensions: Block dimensions
    :type  block_dimensions: int

    :return: Output binary HTJ2K codestream
    :rtype:  bytes
    """
    js_module = await js_package.js_module
    web_worker = js_resources.web_worker

    kwargs = {}
    if decompositions:
        kwargs["decompositions"] = to_js(decompositions)
    if not_reversible:
        kwargs["notReversible"] = to_js(not_reversible)
    if quantization_step:
        kwargs["quantizationStep"] = to_js(quantization_step)
    if progression_order:
        kwargs["progressionOrder"] = to_js(progression_order)
    if tile_size:
        kwargs["tileSize"] = to_js(tile_size)
    if tile_offset:
        kwargs["tileOffset"] = to_js(tile_offset)
    if block_dimensions:
        kwargs["blockDimensions"] = to_js(block_dimensions)

    outputs = await js_module.encode(to_js(image), webWorker=web_worker, noCopy=True, **kwargs)

    output_web_worker = None
    output_list = []
    outputs_object_map = outputs.as_object_map()
    for output_name in outputs.object_keys():
        if output_name == 'webWorker':
            output_web_worker = outputs_object_map[output_name]
        else:
            output_list.append(to_py(outputs_object_map[output_name]))

    js_resources.web_worker = output_web_worker

    if len(output_list) == 1:
        return output_list[0]
    return tuple(output_list)



# Generated file. To retain edits, remove this comment.

"""itkwasm-htj2k-emscripten: Wasm-SIMD accelerated decoding and encoding of High-throughput JPEG2000 images. Emscripten implementation."""

from .decode_async import decode_async
from .encode_async import encode_async

from ._version import __version__
